import { Injectable } from "@angular/core";
import { MenuItem } from "primeng/api";
import { BehaviorSubject } from "rxjs";
import { AuthRoles } from "../../models/auth/auth-roles.enum";
import { AuthService } from "../auth/auth.service";
import { TranslateService } from "@ngx-translate/core";
import { NavigationEnd, Router } from "@angular/router";
import { ThemeService } from "../theme/theme.service";
import { Server } from "../../models/data/server.model";
import { UserDTO } from "../../models/auth/auth-user.dto";

@Injectable({
  providedIn: "root"
})
export class SidebarService {

  isSidebarOpen: boolean = true;
  menuItems$ = new BehaviorSubject<MenuItem[]>(new Array<MenuItem>());
  server$ = new BehaviorSubject<Server | null>(null);

  constructor(
    private themeService: ThemeService,
    private authService: AuthService,
    private translateService: TranslateService,
    private router: Router
  ) {
    this.themeService.isSidebarOpen$.subscribe(value => {
      this.isSidebarOpen = value;
      this.setMenu();
    });

    this.translateService.onLangChange.subscribe(_ => {
      this.setMenu();
    });

    this.server$.subscribe(_ => {
      this.setMenu();
    });

    this.router.events.subscribe(event => {
      if (!(event instanceof NavigationEnd)) {
        return;
      }
      if (!event.url.startsWith("/server/") && this.server$.value) {
        this.setServer(null);
      }
    });
  }

  setServer(server: Server | null) {
    if (server?.id != this.server$.value?.id) {
      this.server$.next(server);
    }
  }

  setMenu() {
    this.authService.hasUserPermission(AuthRoles.Admin).then(async hasPermission => {
      let menuItems: MenuItem[] = [
        { label: this.isSidebarOpen ? this.translateService.instant("sidebar.dashboard") : "", icon: "pi pi-th-large", routerLink: "dashboard" }
      ];

      const serverMenu = {
        label: this.isSidebarOpen ? this.server$.value?.name : "", icon: "pi pi-server", items: [
          { label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.dashboard") : "", icon: "pi pi-th-large", routerLink: `server/${this.server$.value?.id}` },
          { label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.profile") : "", icon: "pi pi-user", routerLink: `server/${this.server$.value?.id}/profile` }
          // { label: this.isSidebarOpen ? this.translateService.instant("sidebar.members") : "", icon: "pi pi-users", routerLink: "server/members" }
        ]
      };
      if (this.server$.value) {
        let authUser = await this.authService.getLoggedInUser();
        let user: UserDTO | null = authUser?.users?.find(u => u.server == this.server$.value?.id) ?? null;

        if (user?.isAdmin) {
          serverMenu.items.push(
            { label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.members") : "", icon: "pi pi-users", routerLink: `server/${this.server$.value?.id}/members` }
          );
        }

        menuItems.push(serverMenu);
      } else if (menuItems.find(x => x.icon == "pi pi-server")) {
        menuItems.splice(menuItems.indexOf(serverMenu), 1);
      }

      if (hasPermission) {
        menuItems.push(
          {
            label: this.isSidebarOpen ? this.translateService.instant("sidebar.administration") : "", icon: "pi pi-cog", items: [
              { label: this.isSidebarOpen ? this.translateService.instant("sidebar.config") : "", icon: "pi pi-cog", routerLink: "/admin/settings" },
              { label: this.isSidebarOpen ? this.translateService.instant("sidebar.auth_user_list") : "", icon: "pi pi-user-edit", routerLink: "/admin/users" }
            ]
          }
        );
      }
      this.menuItems$.next(menuItems);
    });
  }
}
